#ifndef BatchLab_ChainedTuple_h
#define BatchLab_ChainedTuple_h

// Inheritance :
#include <BatchLab/Core/BaseTuple.h>

namespace BatchLab {

class ChainedTuple 
:public BaseTuple
{
public: //AIDA/ITuple
  //fill :
  virtual bool fill(int,double);
  virtual bool fill(int,float);
  virtual bool fill(int,int);
  virtual bool fill(int,short);
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual bool fill(int,long);
#else
  virtual bool fill(int,AIDA::int64);
  virtual bool fill(int,unsigned char);
#endif
  virtual bool fill(int,char);
  virtual bool fill(int,bool);
  virtual bool fill(int,const std::string&);
  virtual bool fill(int,const AIDA::ITupleEntry&);
  virtual bool fill(const std::vector<double>&);
  virtual bool fill(const std::vector<float>&);
  //get :
  virtual double getDouble(int) const;
  virtual float getFloat(int) const;
  virtual int getInt(int) const;
  virtual short getShort(int) const;
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual long getLong(int) const;
#else
  virtual AIDA::int64 getLong(int) const;
#endif
  virtual char getChar(int) const;
  virtual unsigned char getByte(int) const;
  virtual bool getBoolean(int) const;
  virtual std::string getString(int) const;
  virtual const AIDA::ITupleEntry* getObject(int) const;
  //else :
  virtual int rows() const;
  virtual bool addRow();
  virtual void* cast(const std::string&) const;
  virtual const AIDA::ITuple* getTuple(int) const;
  virtual AIDA::ITuple* getTuple(int);
  virtual const AIDA::ITuple* findTuple(int) const; //AIDA-v3r3p0
  virtual AIDA::ITuple* findTuple(int); //AIDA-v3r3p0
  //Override BaseTuple :
  virtual int findColumn(const std::string&) const;
  virtual int columns() const;
  virtual std::string columnName(int) const;
  virtual std::vector<std::string>  columnNames() const;
  virtual std::string columnType(int) const;
  virtual std::vector<std::string>  columnTypes() const;
  virtual void resetRow();
  virtual void reset();
  virtual double columnMin(int) const;
  virtual double columnMax(int) const;
  virtual double columnMean(int) const;
  virtual double columnRms(int) const;
public: //Slash::Store::IObject
  virtual Slash::Store::IObject* copy() const;
  virtual std::string storeClassName() const;
public: //IVisited
  virtual bool visit(Slash::Store::IConstVisitor&) const;
public:
  ChainedTuple(Slash::Core::IWriter&,
               const std::string&,
               const std::string&,
               const std::vector<AIDA::ITuple*>&);
  ChainedTuple(const ChainedTuple&);
  virtual ~ChainedTuple();
private:
  bool findRow(int,AIDA::ITuple*&,int&) const;
private:
  std::vector<AIDA::ITuple*> fTuples;
};

}

#endif


