#ifndef Lib_A2s_h
#define Lib_A2s_h

// Inheritance :
#include <Lib/Array.h>

namespace Lib {

template <class T>
class A2 : public Array<T> {
public:
  A2():Array<T>(1,2){}
  A2(const A2& aFrom):Array<T>(aFrom){}
  A2(const T aVec[2]):Array<T>(1,2) {
    Array<T>::fVector[0] = aVec[0];
    Array<T>::fVector[1] = aVec[1];
  }
  A2(const T& a0,const T& a1):Array<T>(1,2) {
    Array<T>::fVector[0] = a0;
    Array<T>::fVector[1] = a1;
  }
  virtual ~A2() {}
public:
  inline T value0() const { return Array<T>::fVector[0];}
  inline T value1() const { return Array<T>::fVector[1];}

  inline void setValue0(const T& aValue) { Array<T>::fVector[0] = aValue;}
  inline void setValue1(const T& aValue) { Array<T>::fVector[1] = aValue;}

  inline void setValue(const T& a1,const T& a2) {
    Array<T>::fVector[0] = a1;
    Array<T>::fVector[1] = a2;
  }
  inline void setValue(const T aV[2]) {
    Array<T>::fVector[0] = aV[0];
    Array<T>::fVector[1] = aV[1];
  }
  inline void value(T& a1,T& a2) const {
    a1 = Array<T>::fVector[0];
    a2 = Array<T>::fVector[1];
  }    
};

#define A22_D22(a_11,a_12,a_21,a_22)  ((a_11) * (a_22) - (a_21) * (a_12))

template <class T>
class A22 : public ADD<T> {
public:
  A22():ADD<T>(2){}
  A22(const A22& aFrom):ADD<T>(aFrom){}
  virtual ~A22() {}
  inline A22& operator=(const A22& aFrom) {
    ADD<T>::operator=(aFrom);
    return *this;
  }

#define A22_SET_VALUE(aName,a1,a2) \
  inline void aName(const T& aValue) { Array<T>::fVector[a1 + a2 * 2] = aValue;}

  A22_SET_VALUE(setValue00,0,0)
  A22_SET_VALUE(setValue10,1,0)

  A22_SET_VALUE(setValue01,0,1)
  A22_SET_VALUE(setValue11,1,1)

#undef A22_SET_VALUE

#define A22_GET_VALUE(aName,a1,a2) \
  inline T aName() const { return Array<T>::fVector[a1 + a2 * 2];}

  A22_GET_VALUE(value00,0,0)
  A22_GET_VALUE(value10,1,0)

  A22_GET_VALUE(value01,0,1)
  A22_GET_VALUE(value11,1,1)

#undef A22_GET_VALUE

  inline virtual T determinant() const {
    return value00() * value11() - value10() * value01();
  }

  inline bool invert(A22<T>& aResult) const {
    T deter = determinant();
    if(deter==Array<T>::zero()) return false;
    aResult.setValue00( value11());
    aResult.setValue11( value00());
    aResult.setValue01(-value01());
    aResult.setValue10(-value10());
    aResult.multiply(1./deter);  
    return true;
  }

};

}

template <class T>
Lib::A22<T> operator-(const Lib::A22<T>& a1,const Lib::A22<T>& a2) {
  Lib::A22<T> res(a1);
  res.subtract(a2);
  return res;
}
template <class T>
Lib::A22<T> operator+(const Lib::A22<T>& a1,const Lib::A22<T>& a2) {
  Lib::A22<T> res(a1);
  res.add(a2);
  return res;
}
template <class T>
Lib::A22<T> operator*(const Lib::A22<T>& a1,const Lib::A22<T>& a2) {
  Lib::A22<T> res;
  a1.mxmul(a2,res);
  return res;
}

namespace Lib {

template <class T>
A22<T> commutator(const A22<T>& a1,const A22<T>& a2) {
  return a1*a2-a2*a1;
}
template <class T>
A22<T> antiCommutator(const A22<T>& a1,const A22<T>& a2) {
  return a1*a2+a2*a1;
}

//////////////////////////////////////////////////////////////////////////////
/// Some common real matrices ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

typedef A22<double> A22d;

//  1  0
//  0  1
class A22d_I : public A22d {
public:
 A22d_I();
};

//////////////////////////////////////////////////////////////////////////////
/// Some common complex matrices /////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
typedef A22< std::complex<double> > A22z;

class A22z_I : public A22z {
public:
 A22z_I();
};

//  Pauli matrices :
//  Pauli1  Pauli2  Pauli3
//    0  1    0 -i    1  0
//    1  0    i  0    0 -1
class A22z_Pauli1 : public A22z {
public:
 A22z_Pauli1();
};
class A22z_Pauli2 : public A22z {
public:
 A22z_Pauli2();
};
class A22z_Pauli3 : public A22z {
public:
 A22z_Pauli3();
};

//  Epsilon :
//    0  1
//   -1  0
class A22z_Epsilon : public A22z {
public:
 A22z_Epsilon();
};

}

#endif


